@@ -54,6 +54,7 @@ class Controller:
     _VALID_FAN_MODES = {
         "disabled": [Fan.LOW, Fan.MED, Fan.HIGH],
         "unknown": [Fan.LOW, Fan.MED, Fan.HIGH, Fan.TOP, Fan.AUTO],
+        "4-speed": [Fan.LOW, Fan.MED, Fan.HIGH, Fan.TOP, Fan.AUTO],
         "3-speed": [Fan.LOW, Fan.MED, Fan.HIGH, Fan.AUTO],
         "2-speed": [Fan.LOW, Fan.HIGH, Fan.AUTO],
         "var-speed": [Fan.LOW, Fan.MED, Fan.HIGH, Fan.AUTO],
@@ -220,7 +221,7 @@ class Controller:
             AttributeError: On setting if the argument value is not valid
         """
         if value not in self.fan_modes:
-            raise AttributeError("Fan mode {} not allowed".format(value.value))
+            raise AttributeError(f"Fan mode {value.value} not allowed")
         await self._set_system_state(
             "SysFan",
             "SystemFAN",
@@ -242,7 +243,7 @@ class Controller:
         time = int(value)
         if time < 0 or time > 120 or time % 30 != 0:
             raise AttributeError(
-                'Invalid Sleep Timer "{}", must be divisible by 30'.format(value)
+                f'Invalid Sleep Timer "{value}", must be divisible by 30'
             )
         await self._set_system_state("SleepTimer", "SleepTimer", value, time)
 
@@ -291,11 +292,9 @@ class Controller:
                 Can still be set even if the mode isn't appropriate.
         """
         if value % 0.5 != 0:
-            raise AttributeError(
-                "SetPoint '{}' not rounded to nearest 0.5".format(value)
-            )
+            raise AttributeError(f"SetPoint '{value}' not rounded to nearest 0.5")
         if value < self.temp_min or value > self.temp_max:
-            raise AttributeError("SetPoint '{}' is out of range".format(value))
+            raise AttributeError(f"SetPoint '{value}' is out of range")
         await self._set_system_state("Setpoint", "UnitSetpoint", value, str(value))
 
     @property
@@ -399,9 +398,7 @@ class Controller:
 
     async def _refresh_zone_group(self, group: int, notify: bool = True):
         assert group in [0, 4, 8]
-        zone_data_part = await self._get_resource(
-            "Zones{0}_{1}".format(group + 1, group + 4)
-        )
+        zone_data_part = await self._get_resource(f"Zones{group + 1}_{group + 4}")
 
         for i in range(min(len(self.zones) - group, 4)):
             zone_data = zone_data_part[i]
@@ -422,7 +419,7 @@ class Controller:
     async def _set_system_state(self, state, command, value, send=None):
         if send is None:
             send = value
-        await self._send_command_async(command, send)
+        await self._send_command_async(command, {command: send})
 
         # Update state and trigger rescan
         self._system_settings[state] = value
@@ -473,7 +470,7 @@ class Controller:
         try:
             session = self._discovery.session
             async with self._sending_lock, session.get(
-                "http://%s/%s" % (self.device_ip, resource),
+                f"http://{self.device_ip}/{resource}",
                 timeout=Controller.REQUEST_TIMEOUT,
             ) as response:
                 try:
@@ -497,78 +494,59 @@ class Controller:
         loop = asyncio.get_running_loop()
         on_complete = loop.create_future()
         device_ip = self.device_ip
-        response = []
 
         class _PostProtocol(asyncio.Protocol):
+            def __init__(self) -> None:
+                self.response = bytearray()
+
             def connection_made(self, transport):
-                body = json.dumps({command: data}).encode()
+                body = json.dumps(data).encode("latin_1")
                 header = (
-                    "POST /"
-                    + command
-                    + " HTTP/1.1\r\n"
-                    + "Host: "
-                    + device_ip
-                    + "\r\n"
-                    + "Content-Length: "
-                    + str(len(body))
-                    + "\r\n"
-                    + "\r\n"
+                    f"POST /{command} HTTP/1.1\r\n"
+                    f"Host: {device_ip}\r\n"
+                    f"Content-Length: {str(len(body))}\r\n"
+                    "\r\n"
                 ).encode()
-                _LOG.debug("Writing message to " + device_ip + body.decode())
+                _LOG.debug("Writing message to %s", device_ip)
                 transport.write(header + body)
-                # pylint: disable=attribute-defined-outside-init
-                self.transport = transport
 
             def data_received(self, data):
-                response.append(data.decode())
+                self.response += data
 
             def eof_received(self):
-                self.transport.close()
-                lines = "".join(response).split("\r\n")
-                if not lines:
+                full = self.response.decode("latin_1")
+                if not full:
+                    on_complete.set_exception(RuntimeError("Empty HTTP Response"))
                     return
-                parts = lines[0].split(" ")
-                if len(parts) != 3:
+                header, _ = full.split("\r\n", 1)
+                parts = header.split(" ", 2)
+                if len(parts) < 3 or parts[0] != "HTTP/1.1":
+                    on_complete.set_exception(RuntimeError("Invalid HTTP Response"))
                     return
-                if int(parts[1]) != 200:
+                if parts[1] != "200":
                     on_complete.set_exception(
-                        aiohttp.ClientResponseError(
-                            None, None, status=int(parts[1]), message=parts[2]
+                        aiohttp.ClientError(
+                            f"Unable to connect to: http://{device_ip}/{command}"
+                            f" response={parts[1]} message={parts[2]}"
                         )
                     )
-                elif len(lines) > 2 and len(lines[-2]) == 0:
-                    result = lines[-1]
-                    if len(result) > 7 and result[:6] == "{ERROR":
-                        on_complete.set_exception(
-                            ConnectionError(f"Server returned error state {result}")
-                        )
-                        return
-                    if len(result) > 4 and result[-4:] == "{OK}":
-                        result = result[:-4]
-                    on_complete.set_result(result)
-                else:
-                    on_complete.set_result(None)
+                    return
+                _, content = full.split("\r\n\r\n", 1)
+                on_complete.set_result(content)
 
         # The server doesn't tolerate multiple requests in fly concurrently
         try:
-            async with self._sending_lock, timeout(Controller.REQUEST_TIMEOUT) as timer:
-                # pylint: disable=unnecessary-lambda
-                transport, _ = await loop.create_connection(  # type: ignore  # noqa: E501
-                    lambda: _PostProtocol(), self.device_ip, 80
-                )
-
-                # wait for response to be recieved.
+            async with self._sending_lock, timeout(Controller.REQUEST_TIMEOUT):
+                await loop.create_connection(_PostProtocol, self.device_ip, 80)
                 await on_complete
 
-            if timer.expired:
-                if transport:
-                    transport.close()
-                raise asyncio.TimeoutError()
-
-            return on_complete.result()
-
-        except ConnectionError:
-            raise
+            result = on_complete.result()
         except (OSError, asyncio.TimeoutError, aiohttp.ClientError) as ex:
             self._failed_connection(ex)
             raise ConnectionError("Unable to connect to controller") from ex
+
+        if len(result) >= 7 and result[:6] == "{ERROR":
+            raise ConnectionError(f"Server returned error state {result}")
+        if len(result) >= 4 and result[-4:] == "{OK}":
+            result = result[:-4]
+        return result
